//---------------------------------------------------
//                 Connect Four
//---------------------------------------------------

#include <gamekit_2_1.h>




void setup(){
  gamekit.Begin();
  Serial.begin(9600);
}



//---------------------------------------------------
//             Random Number Generator
//---------------------------------------------------
long denominator = 172373;
long absolute    = 572665;
long multiplier =  1146;
long summand     = 568781;
double generate_rand_double() {
    absolute = (multiplier*absolute+summand+(10*gamekit.get_systemcounter()%denominator) )%denominator;
    return float(absolute)/denominator;
}


//---------------------------------------------------
//                Melody Functions
//---------------------------------------------------

int melody_1[56][2] = {{1,1},{1,3},{1,5},{1,6}, {1,8},{1,8},{1,8},{1,8}, {1,10},{1,10},{1,10},{1,10}, {1,8},{1,8},{1,8},{1,8}, {1,10},{1,10},{1,10},{1,10}, {1,8},{1,8},{1,8},{1,8}, {1,6},{1,6},{1,6},{1,6}, {1,5},{1,5},{1,5},{1,5}, {1,3},{1,3},{1,3},{1,3}, {1,1},{1,1},{1,1},{1,1}, {1,0},{1,0},{1,0},{1,0}, {1,0},{1,0},{1,0},{1,0}, {1,0},{1,0},{1,0},{1,0}, {1,0},{1,0},{1,0},{1,0} };
int melody_length_1 = 55;
int melody_position_1 = 0;
int melody_speed_1 = 200;

int melody_2[110][2] = {{2,17},{1,12},{1,13},{2,15},{1,13},{1,12}  ,{2,10},{1,10},{1,13},{2,17},{1,15},{1,13}, {3,12},{1,13},{2,15},{2,17} ,{2,13},{2,10},{4,10} ,{1,0},{2,15},{1,18},{2,22},{1,20},{1,18} ,{3,17},{1,13},{2,17},{1,15},{1,13} ,{2,12},{1,12},{1,13},{2,15},{2,17}  ,{2,13},{4,10},{2,0},    {2,17},{1,12},{1,13},{2,15},{1,13},{1,12}  ,{2,10},{1,10},{1,13},{2,17},{1,15},{1,13}, {3,12},{1,13},{2,15},{2,17} ,{2,13},{2,10},{2,10},{2,0} ,{1,0},{2,15},{1,18},{2,22},{1,20},{1,18} ,{3,17},{1,13},{2,17},{1,15},{1,13} ,{2,12},{1,12},{1,13},{2,15},{2,17}  ,{2,13},{2,10},{2,10},{2,0} ,{2,5},{2,5},{2,1},{2,1} ,{2,3},{2,3},{2,-1},{2,-1} ,{2,1},{2,1},{2,-3},{2,-3}, {2,-4},{2,-4},{2,-1},{2,0}, {2,5},{2,5},{2,1},{2,1}, {2,3},{2,3},{2,-1},{2,-1}, {2,1},{2,5},{2,10},{2,10}, {2,9},{2,9},{2,0},{2,0} };
int melody_length_2 = 109;
int melody_position_2 = 0;
int melody_speed_2 = 180;

int sound_volume = 1;

void play_next_melody_tone(int melody_no) {
    if (melody_no==1) {
        play_tone(melody_1[melody_position_1][1], melody_1[melody_position_1][0]*melody_speed_1, sound_volume);
        melody_position_1++;
        if (melody_position_1>melody_length_1) {
            melody_position_1=0;
        }      
    }
    if (melody_no==2) {
        play_tone(melody_2[melody_position_2][1], melody_2[melody_position_2][0]*melody_speed_2, sound_volume);
        melody_position_2++;
        if (melody_position_2>melody_length_2) {
            melody_position_2=0;
        }
    }
}

void play_frequency(int frequency_, int duration_, int loudness_){
    //loudness: 0=no sound, 1=silent, 2=loud
        if (loudness_==0) {
            //do nothing but delay
            delay(duration_);
        } else {
        if (loudness_==1) {
            gamekit.play_tone(frequency_, duration_, 0);
        } else {
            gamekit.play_tone(frequency_, duration_, 1);
        }
    }
}

void play_tone(int distance_to_C_, int duration_, int loudness_){
    if (distance_to_C_!=0) {
        play_frequency(246  *  pow(float(2), float(distance_to_C_)/12.0), duration_, loudness_);
    } else {
        play_frequency(264, duration_, 0);
    }
}









//---------------------------------------------------
//                   The Game
//---------------------------------------------------

    int active_player = 1;
    int color[5] = {0, 1, 15, 1, 15};      //No Player, Player I, Player II, Player I selctor, Player II selector
    int position = 0;
    int ai_player = 2;                     //Who is the AI player? 0=none, 1=player I, 2=Player II
    int nonai_player = 1;                  //Only neede of one ai player exists
    int ai_handycap = 1;                   //1=best play , .... , 4=worst play
    
    int field[5][7];
    int field_save[5][7];
    int win_stones[4][2];                  //Stones that connect four. [4 stones][{x,y}]

    int pixel_on = 0;         //0=off, 100=on, -200=disable blinking

    int FallCount = 0;        

    int GamePhase = 0;        //1=Selection, 2=Falling Sequence, 3=Winning Sequence


void move_to_next_free_position(int first_direction) {    //first_direction could be: -1 for left or 1 for right
    for (int position_to_check=position; (position_to_check<=6)&&(position_to_check>=0); position_to_check=position_to_check+first_direction) {
        if (field[0][position_to_check]==0) {
            position=position_to_check;
            break;
        }
    }
    for (int position_to_check=position; (position_to_check<=6)&&(position_to_check>=0); position_to_check=position_to_check+(-1*first_direction)) {
        if (field[0][position_to_check]==0) {
            position=position_to_check;
            break;
        }
    }
}

int Check_Win() {
    //horizontal
    for (int y=0; y<=4; y++) {
        for (int x=0; x<=3; x++) {
            if (field[y][x]!=0) {
                if ( (field[y][x]==field[y][x+1]) && (field[y][x]==field[y][x+2]) && (field[y][x]==field[y][x+3]) ) {
                    Serial.print("Player "); Serial.print(field[y][x]); Serial.print(" wins (horizontal): "); Serial.print(y); Serial.print(", "); Serial.println(x);
                    win_stones[0][0] = y;   win_stones[0][1] = x;         //The first win stone
                    win_stones[1][0] = y;   win_stones[1][1] = x+1;
                    win_stones[2][0] = y;   win_stones[2][1] = x+2;
                    win_stones[3][0] = y;   win_stones[3][1] = x+3;       //The last win stone
                    return field[y][x];
                }
            }
        }
    }
    //vertical
    for (int y=0; y<=1; y++) {
        for (int x=0; x<=6; x++) {
            if (field[y][x]!=0) {
                if ( (field[y][x]==field[y+1][x]) && (field[y][x]==field[y+2][x]) && (field[y][x]==field[y+3][x]) ) {
                    Serial.print("Player "); Serial.print(field[y][x]); Serial.print(" wins (vertical): "); Serial.print(y); Serial.print(", "); Serial.println(x);
                    win_stones[0][0] = y;   win_stones[0][1] = x;         //The first win stone
                    win_stones[1][0] = y+1; win_stones[1][1] = x;
                    win_stones[2][0] = y+2; win_stones[2][1] = x;
                    win_stones[3][0] = y+3; win_stones[3][1] = x;       //The last win stone
                    return field[y][x];
                }
            }
        }
    }
    //diagonal (\)
    for (int y=0; y<=1; y++) {
        for (int x=0; x<=3; x++) {
            if (field[y][x]!=0) {
                if ( (field[y][x]==field[y+1][x+1]) && (field[y][x]==field[y+2][x+2]) && (field[y][x]==field[y+3][x+3]) ) {
                    Serial.print("Player "); Serial.print(field[y][x]); Serial.print(" wins (diagonal \\): "); Serial.print(y); Serial.print(", "); Serial.println(x);
                    win_stones[0][0] = y;   win_stones[0][1] = x;         //The first win stone
                    win_stones[1][0] = y+1; win_stones[1][1] = x+1;
                    win_stones[2][0] = y+2; win_stones[2][1] = x+2;
                    win_stones[3][0] = y+3; win_stones[3][1] = x+3;       //The last win stone
                    return field[y][x];
                }
            }
        }
    }
    //diagonal (/)
    for (int y=0; y<=1; y++) {
        for (int x=3; x<=6; x++) {
            if (field[y][x]!=0) {
                if ( (field[y][x]==field[y+1][x-1]) && (field[y][x]==field[y+2][x-2]) && (field[y][x]==field[y+3][x-3]) ) {
                    Serial.print("Player "); Serial.print(field[y][x]); Serial.print(" wins (diagonal /): "); Serial.print(y); Serial.print(", "); Serial.println(x);
                    win_stones[0][0] = y;   win_stones[0][1] = x;         //The first win stone
                    win_stones[1][0] = y+1;   win_stones[1][1] = x-1;
                    win_stones[2][0] = y+2;   win_stones[2][1] = x-2;
                    win_stones[3][0] = y+3;   win_stones[3][1] = x-3;       //The last win stone
                    return field[y][x];
                }
            }
        }
    }
    
    return 0;
  
}




        int zug = 0;
        int zugliste[10] = {4, 3, 4, 1, 0, 0, 6, 4, 3, 4};

void loop(){
  
    if (GamePhase == 0) {    //Settings
        //Sound Volume
        if (gamekit.button_pressed(butt_RIGHT)) {
            sound_volume++;
            if (sound_volume==3) {
                sound_volume = 0;
            }
        }
        for (int vol=0; vol<=2; vol++) {
            if (sound_volume>=vol) { 
                gamekit.set_pixel(2, vol, 8);
            } else {
                gamekit.set_pixel(2, vol, 0);
            }  
        }
        
        //AI Player
        if (gamekit.button_pressed(butt_UP)) {
            ai_player--;
            if (ai_player==-1) {
                ai_player = 2;
            }
        }
        switch (ai_player) {
            case 0:
                gamekit.set_pixel(0, 0, color[1]); gamekit.set_pixel(0, 1, color[2]);
                nonai_player = 1;
            break;
            case 1:
                nonai_player = 2;
                gamekit.set_pixel(0, 0, 0); gamekit.set_pixel(0, 1, color[2]);
            break;
            case 2:
                gamekit.set_pixel(0, 0, color[1]); gamekit.set_pixel(0, 1, 0);
                nonai_player = 1;
            break;
        }
      
        //AI power
        if (gamekit.button_pressed(butt_LEFT)) {
            ai_handycap--;
            if (ai_handycap==0) {
                ai_handycap = 4;
            }
        }
        for (int pow=0; pow<=4; pow++) {
            if (4-ai_handycap>=pow) { 
                gamekit.set_pixel(4, pow, 8);
            } else {
                gamekit.set_pixel(4, pow, 0);
            }  
        }
      
      
      
        play_next_melody_tone(2);
      
        if ((gamekit.button_pressed(butt_FUNCA)) || (gamekit.button_pressed(butt_FUNCB))) {
            gamekit.set_pixel(0, 0, 0); gamekit.set_pixel(0, 1, 0);
            gamekit.set_pixel(2, 0, 0); gamekit.set_pixel(2, 1, 0); gamekit.set_pixel(2, 2, 0);
            gamekit.set_pixel(4, 0, 0); gamekit.set_pixel(4, 1, 0); gamekit.set_pixel(4, 2, 0); gamekit.set_pixel(4, 3, 0);
            GamePhase = 1;
        }
    }
    





    if (GamePhase == 1) {    //Selection
        if (active_player != ai_player) {   //active player is human
            if (gamekit.button_pressed(butt_LEFT)) {
                gamekit.set_pixel(0, position, color[field[0][position]]);
                if (position>0) {
                    position--;
                }
                move_to_next_free_position(-1);
            }
            if (gamekit.button_pressed(butt_RIGHT)) {
                gamekit.set_pixel(0, position, color[field[0][position]]);
                if (position<6) {
                    position++;
                }
                move_to_next_free_position(1);
            }
            if ((gamekit.button_pressed(butt_DOWN)) || (gamekit.button_pressed(butt_FUNCA)) || (gamekit.button_pressed(butt_FUNCB)) ) {
                GamePhase = 2;
            }      
    
            gamekit.set_pixel(0, position, min(pixel_on,color[active_player+2]));
            if (pixel_on==0) {
                pixel_on = 100;
            } else {
                if (pixel_on==100) {
                    pixel_on = 0;
                }
            }
            delay(200);
        } else {        //active player is AI
           
            //Quick check if other player could win in one step
            int decision = 0;
            int row = 0;
            boolean QuickCheckSucceded = false;
            for (int col=0; col<=6; col++) {
                if (field[0][col]==0) {
                    row = 0;
                    do {    
                        if ( (field[row+1][col]!=0) || (row==4) ) {
                            break;
                        }
                        row++;
                    } while (true);
                    field[row][col]=nonai_player;
                    if (Check_Win()==nonai_player) {
                        decision = col;
                        QuickCheckSucceded=true;
                    }                
                    field[row][col]=0;
                }
            }

            //Quick check if ai player could win in one step
            for (int col=0; col<=6; col++) {
                if (field[0][col]==0) {
                    row = 0;
                    do {    
                        if ( (field[row+1][col]!=0) || (row==4) ) {
                            break;
                        }
                        row++;
                    } while (true);
                    field[row][col]=ai_player;
                    if (Check_Win()==ai_player) {
                        decision = col;
                        QuickCheckSucceded=true;
                    }                
                    field[row][col]=0;
                }
            }
           
           
           
           
            if (QuickCheckSucceded==false) {
            int row[4];
            double points[4] = {0, 0};
            double best_points[4] = {-1000, 1000, -1000, 1000};
            boolean WinSwitch = false;
            //  -----  ai step 1  -----            
            for (int col=0; col<=6; col++) {
                //Serial.println(col);
                Serial.print(0); Serial.print(" "); Serial.print(col); Serial.print(" "); Serial.print(field[0][col]);
                if (field[0][col]==0) {
                    Serial.print(" --> in   ");
                    row[0] = 0;
                    do {    
                        if ( (field[row[0]+1][col]!=0) || (row[0]==4) ) {
                            break;
                        }
                        row[0]++;
                    } while (true);
                    field[row[0]][col]=ai_player;
                    WinSwitch=false;
                    if (Check_Win()==0) {
                        WinSwitch=false;
                    } else {
                        WinSwitch=true;
                        points[0]=999;
                    }
                    if (WinSwitch==false) {
                        //  -----  non ai step 1  -----
                        for (int col2=0; col2<=6; col2++) {  
                            gamekit.set_pixel(0, col, min(color[ai_player+2], 100*((col2+1)%2)) );  //show selection pixel in coloumn "col" if "col2" is eaven
                            //Serial.print("  "); Serial.println(col2);
                            if (field[0][col2]==0) {
                                row[1] = 0;
                                do {    
                                    if ( (field[row[1]+1][col2]!=0) || (row[1]==4) ) {
                                        break;
                                    }
                                    row[1]++;
                                } while (true);
                                field[row[1]][col2]=nonai_player;
                                WinSwitch=false;
                                if (Check_Win()==0) {
                                    WinSwitch=false;
                                } else {
                                    WinSwitch=true;
                                    points[1]=-999;
                                }
                                if (WinSwitch==false) {
                                //  -----  ai step 2  -----
                                    for (int col3=0; col3<=6; col3++) {  
                                        //Serial.print("    "); Serial.println(col3); 
                                        if (field[0][col3]==0) {
                                            row[2] = 0;
                                            do {    
                                                if ( (field[row[2]+1][col3]!=0) || (row[2]==4) ) {
                                                    break;
                                                }
                                                row[2]++;
                                            } while (true);
                                            field[row[2]][col3]=ai_player;
                                            WinSwitch=false;
                                            if (Check_Win()==0) {
                                                WinSwitch=false;
                                            } else {
                                                WinSwitch=true;
                                                points[2]=999;
                                            }
                                            if (WinSwitch==false) {
                                                for (int col4=0; col4<=6; col4++) {  //  -----  non ai step 2  -----
                                                    //Serial.print("      "); Serial.print(col4);
                                                    if (field[0][col4]==0) {
                                                      
                                                        row[3] = 0;
                                                        do {    
                                                            if ( (field[row[3]+1][col4]!=0) || (row[3]==4) ) {
                                                                break;
                                                            }
                                                            row[3]++;
                                                        } while (true);
                                                        field[row[3]][col4]=nonai_player;
                                                   
                                                        points[3] = evaluate_board();
                                                        //Serial.print("  --> "); Serial.print(points[3]);
                            
                                                        field[row[3]][col4]=0;
                                                        if (points[3]<best_points[3]) {
                                                            best_points[3] = points[3];
                                                            points[2] = points[3];
                                                        }
                                                    }
                                                }
                                            }
                                            //Serial.println("");
                                            best_points[3] = 999;
                              
                                            field[row[2]][col3]=0;
                                            if (points[2]>best_points[2]) {
                                                best_points[2] = points[2];
                                                points[1] = points[2];
                                            }
                                            
                                        }
                                    }
                                }
                                //Serial.print(" --> "); Serial.print(points[3]); Serial.println(" || ");
                                field[row[1]][col2]=0;
                                if (points[1]<best_points[1]) {
                                    best_points[1] = points[1];
                                    points[0] = points[1];
                                }
                                best_points[2] = -999;
                                
                            }
                        }
                    }
                    best_points[1] = 999;                    
                    //points = evaluate_board();
                    //Serial.print(" | ----> "); Serial.print(points[0]); Serial.println(" || ");
                    field[row[0]][col]=0;
                    points[0] = points[0] + (generate_rand_double() * pow(ai_handycap,2));      //Disturb the first choice
                    if (points[0]>best_points[0]) {
                        best_points[0] = points[0];
                        decision = col;
                    }
                    gamekit.set_pixel(0, col, 0);
                }
            }
            }
            Serial.println("--------------------------"); 

            GamePhase = 2;
            position = decision;
            
            
        }
    }
    

    if (GamePhase==2) {        //Fall animation
        gamekit.set_pixel(FallCount, position, color[active_player]);
        gamekit.set_pixel(FallCount-1, position, 0);
        play_tone(9-2*FallCount, 200, sound_volume);
        if ( (field[FallCount+1][position]==0) && (FallCount<=3) ) {
            FallCount++;
        } else {
            field[FallCount][position] = active_player;
            FallCount = 0;
            GamePhase = 1;
                move_to_next_free_position(1);
                active_player++;
                if (active_player==3) {
                    active_player = 1;
                }
                if ( Check_Win()!=0 ) {
                    GamePhase=3;
                }
        }
    }


    if (GamePhase==3) {        //Win animation
            gamekit.set_pixel(0, position, 0);    //hide selection point
            melody_position_1 = 0;
            boolean new_game = false;  
            do {
                gamekit.set_pixel(win_stones[0][0], win_stones[0][1], min(pixel_on,color[field[win_stones[0][0]][win_stones[0][1]]]));
                gamekit.set_pixel(win_stones[1][0], win_stones[1][1], min(pixel_on,color[field[win_stones[0][0]][win_stones[0][1]]]));
                gamekit.set_pixel(win_stones[2][0], win_stones[2][1], min(pixel_on,color[field[win_stones[0][0]][win_stones[0][1]]]));
                gamekit.set_pixel(win_stones[3][0], win_stones[3][1], min(pixel_on,color[field[win_stones[0][0]][win_stones[0][1]]]));
                if (pixel_on==0) {
                    pixel_on = 100;
                } else {
                    if (pixel_on == 100) {
                        pixel_on = 0;
                    }
                }
                play_next_melody_tone(1);
                if ( (gamekit.button_pressed(butt_UP)) || (gamekit.button_pressed(butt_DOWN)) || (gamekit.button_pressed(butt_LEFT)) || (gamekit.button_pressed(butt_RIGHT)) || (gamekit.button_pressed(butt_FUNCA)) || (gamekit.button_pressed(butt_FUNCB) ) ) {
                    new_game = true;
                    GamePhase=0;
                    clear_field();
                    melody_position_2 = 0;
                }
            } while (new_game==false);
       
    }
    
    
    //Check if all places are filled
    if ( (field[0][0]!=0) && (field[0][1]!=0) && (field[0][2]!=0) && (field[0][3]!=0) && (field[0][4]!=0) && (field[0][5]!=0) && (field[0][6]!=0) && (field[0][7]!=0) ) {
        boolean new_game = false;
        do {
            play_next_melody_tone(1);
            if ( (gamekit.button_pressed(butt_UP)) || (gamekit.button_pressed(butt_DOWN)) || (gamekit.button_pressed(butt_LEFT)) || (gamekit.button_pressed(butt_RIGHT)) || (gamekit.button_pressed(butt_FUNCA)) || (gamekit.button_pressed(butt_FUNCB) ) ) {
                new_game = true;
                GamePhase=0;
                clear_field();
            }
        } while (new_game==false);
    } 
  

}
























//---------------------------------------------------
//              Board evaluation for AI
//---------------------------------------------------


double evaluate_point(int p_x, int p_y) {

  double points=0;
  double points_sum=0;
  
  if (field[p_y][p_x]==0) {
      return 0;
  }
  int player = field[p_y][p_x];

  int count;    
  
  //horizontal
if (field[p_y][p_x-1]!=player) {
  for (count=0; p_x+count<=6; count++) {
      if (field[p_y][p_x+count]!=player) {
          break;
      }
  }
//  if (count>1) {
//      Serial.print(" (Player "); Serial.print(player); Serial.print(" has horizontal: "); Serial.print(count); Serial.print(" at "); Serial.print(p_y); Serial.print(", "); Serial.print(p_x); Serial.print(")");
//  }
  switch (count) {
    case 2:
      points = 3;
      break;
    case 3:
      points = 8;
      break;
    case 4: 
      points = 99;
  }
  if ((player!=ai_player)&&(count>1)) {
      points = -1*points;
  }
  //Serial.print("--> "); Serial.println(points);
  points_sum = points_sum+points;
  points=0;
}
  //vertical
if (field[p_y-1][p_x]!=player) {
  for (count=0; p_y+count<=4; count++) {
      if (field[p_y+count][p_x]!=player) {
          break;
      }
  }
//  if (count>1) {
//      Serial.print(" (Player "); Serial.print(player); Serial.print(" has vertical: "); Serial.print(count); Serial.print(" at "); Serial.print(p_y); Serial.print(", "); Serial.print(p_x); Serial.print(")");
//  }
  switch (count) {
    case 2:
      points = 2;
      break;
    case 3:
      points = 7;
      break;
    case 4: 
      points = 99;
  }
  if ((player!=ai_player)&&(count>1)) {
      points = -1*points;
  }
  //Serial.print("--> "); Serial.println(points);
  points_sum = points_sum+points;
  points=0;
}

  //diagonal (\)
if (field[p_y-1][p_x-1]!=player) {
  for (count=0; (p_x+count<=6)||(p_y+count<=4); count++) {
      if (field[p_y+count][p_x+count]!=player) {
          break;
      }
  }
//  if (count>1) {
//      Serial.print(" (Player "); Serial.print(player); Serial.print(" has diagonal (\\): "); Serial.print(count); Serial.print(" at "); Serial.print(p_y); Serial.print(", "); Serial.println(p_x); Serial.print(")");
//  }
  switch (count) {
    case 2:
      points = 2;
      break;
    case 3:
      points = 7;
      break;
    case 4: 
      points = 99;
  }
  if ((player!=ai_player)&&(count>1)) {
      points = -1*points;
  }
  //Serial.print("--> "); Serial.println(points);
  points_sum = points_sum+points;
  points=0;
}

  //diagonal (/)
if (field[p_y-1][p_x+1]!=player) {
  for (count=0; (p_x-count>=0)||(p_y+count<=4); count++) {
      if (field[p_y+count][p_x-count]!=player) {
          break;
      }
  }
//  if (count>1) {
//      Serial.print(" (Player "); Serial.print(player); Serial.print(" has diagonal (/): "); Serial.print(count); Serial.print(" at "); Serial.print(p_y); Serial.print(", "); Serial.print(p_x); Serial.print(")");
//  }
  switch (count) {
    case 2:
      points = 2;
      break;
    case 3:
      points = 7;
      break;
    case 4: 
      points = 99;
  }
  if ((player!=ai_player)&&(count>1)) {
      points = -1*points;
  }
  //Serial.print("--> "); Serial.println(points);
  points_sum = points_sum+points;
  points=0;
}

  
  return points_sum;
}



double evaluate_board() {
    double points_board = 0;
    
    for (int y=0; y<=4; y++) {
        for (int x=0; x<=6; x++) {
          
            points_board = points_board + evaluate_point(x, y);
          
        }
    }     
    
    return points_board;
  
}


void save_field() {
    for (int a=0; a<=4; a++) {
        for (int b=0; b<=6; b++) {
            field_save[a][b] = field[a][b];
        }
    }  
}

void restore_field() {
    for (int a=0; a<=4; a++) {
        for (int b=0; b<=6; b++) {
            field[a][b] = field_save[a][b];
        }
    }  
}

void plot_field() {
    for (int a=0; a<=4; a++) {
        for (int b=0; b<=6; b++) {
            Serial.print(field[a][b]); Serial.print(" ");
        }
        Serial.println();
    }  
}

void clear_field() {
    for (int a=0; a<=4; a++) {
        for (int b=0; b<=6; b++) {
            field[a][b]=0;
            gamekit.set_pixel(a, b, 0);
        }
        Serial.println();
    }    
}
